﻿using System;
using System.Collections.Generic;

using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Windows.Forms;

using WindowsOS;

namespace NAVApplication
{
    /*
     * This class represents the ClientUserSettings.config file that the Microsoft Dynamics NAV RoleTailored Client uses when
     * it loads.
     */
    public class ClientUserConfig
    {
        /*
         * The full path and name of the CustomSettings.config file.
         * Windows XP location: 
         *    C:\Documents and Settings\<username>\Local Settings\Application Data\Microsoft\Microsoft Dynamics NAV\ClientUserSettings.config
         * Windows Vista, Windows 7 location:  
         *    C:\Users\<username>\AppData\Local\Microsoft\Microsoft Dynamics NAV\ClientUserSettings.config
         *    
         * If the file does not exist when assigning the value a FileNotFoundException is thrown.
         */
        private string _filename;
        public string filename
        {
            get
            {
                return _filename;
            }
            set
            {
                if (File.Exists(value))
                {
                    _filename = value;
                }
                else
                {
                    throw new FileNotFoundException(value + " could not be found!");
                }
            }
        }

        /*
         * The original contents of the ClientUserSettings.config file. This content is stored because we do not overwrite the
         * file with all of the other options that have been set, which may be different than the default settings.
         */
        private XmlDocument _originalFileContents;
        public XmlDocument originalFileContents { get { return _originalFileContents; } set { _originalFileContents = value; } }

        /*
         * Represents the key / value pairs from the ClientUserSettings.config file.
         * 
         * Same key / value pair form file:
         * <!-- Name of the Microsoft Dynamics NAV Server instance to connect to (for client) or listen on (for server). -->
         * <add key="ServerName" value="CanvasNAV02"></add>
         * 
         * Use the GetValue method below to retrieve the value for one of the keys in the file.
         */
        private Dictionary<string, string> _keyValuePairs;

        /*
         * Constructor
         * @param windowsVersion A custom enumeratio from the WindowsOS namespace that specified the Windows Version
         */
        public ClientUserConfig(WindowsVersion windowsVersion)
        {
            _keyValuePairs = new Dictionary<string, string>();

            try
            {
                switch (windowsVersion)
                {
                    case WindowsVersion.WindowsXP:
                        filename = @"C:\Documents and Settings\" +
                            Environment.UserName + @"\Local Settings\Application Data\Microsoft\Microsoft Dynamics NAV\ClientUserSettings.config";
                        break;
                    case WindowsVersion.WindowsVista:
                    case WindowsVersion.Windows7:
                        filename = @"C:\Users\" +
                            Environment.UserName + @"\AppData\Local\Microsoft\Microsoft Dynamics NAV\ClientUserSettings.config";
                        break;
                    default:
                        throw new Exception("Unable to determine path for " + filename);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + Environment.NewLine + ex.StackTrace,
                    "NAV Application", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            LoadFile();
        }

        /*
         * Method LoadFile
         * 
         * Loads the original contents of the ClientUserSettings.config file
         */
        public void LoadFile()
        {
            try
            {
                if (File.Exists(filename))
                {
                    originalFileContents = new XmlDocument();
                    originalFileContents.Load(filename);
                }
                else
                {
                    throw new FileNotFoundException(filename + " could not be found!");
                }

                XPathNavigator navigator = originalFileContents.CreateNavigator();
                XPathExpression expr = navigator.Compile("/configuration/appSettings/add");
                XPathNodeIterator iterator = navigator.Select(expr);

                string key = string.Empty;
                string value = string.Empty;

                while (iterator.MoveNext())
                {
                    key = iterator.Current.Clone().GetAttribute("key", "");
                    value = iterator.Current.Clone().GetAttribute("value", "");
                    if (!_keyValuePairs.ContainsKey(key))
                    {
                        _keyValuePairs.Add(key, value);
                    }
                }
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message + Environment.NewLine + ex2.StackTrace,
                    "NAV Application", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /*
         * Method OverwriteConfigFile
         * @param server The new server name to connect to
         * @param serverInstance The new Service Tier Service to connect to
         * @param serverPort The port number that the ServiceTier Service is running on
         */
        public void OverwriteConfigFile(string server, string serverInstance, string serverPort)
        {
            try
            {
                XmlDocument fileContentsCopy = originalFileContents;
                XmlElement root = fileContentsCopy.DocumentElement;
                XmlNode node;

                node = root.SelectSingleNode("/configuration/appSettings/add[@key='Server']");
                node.Attributes["value"].Value = server;

                node = root.SelectSingleNode("/configuration/appSettings/add[@key='ServerInstance']");
                node.Attributes["value"].Value = serverInstance;

                node = root.SelectSingleNode("/configuration/appSettings/add[@key='ServerPort']");
                node.Attributes["value"].Value = serverPort;

                fileContentsCopy.Save(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + Environment.NewLine + ex.StackTrace,
                    "NAV Application", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /*
         * Function GetValue
         * @param key Represents the key found in the ClientUserSettings.config file.
         * @return string The value of the key / value pair from the ClientUserSettings.config file
         * 
         * This function will return the value associated with the key passed in. If no key is found null is returned.
         */
        public string GetValue(string key)
        {
            string value;
            if (_keyValuePairs.TryGetValue(key, out value))
            {
                return value;
            }
            else
            {
                return null;
            }
        }
    }
}
