﻿using System.Collections.Generic;
using System.ServiceProcess;
using System;

namespace NAVServiceTier
{
    /*
     * This class represent a single NAV Server that runs one or more NAV Service Tiers.
     */
    public class NAVServer
    {
        /*
         * The name of this server.
         */
        private string _serverName;
        public string serverName { get { return _serverName; } set { _serverName = value; } }
                
        /*
         * A List of all of the NAV Service Tiers running on this server.
         */
        private Dictionary<string, NAVService> _NAVServices;
        public Dictionary<string, NAVService> NAVServices { get { return _NAVServices; } set { _NAVServices = value; } }

        /*
         * Default constructor to instantiate variables.
         */
        public NAVServer()
        {
            NAVServices = new Dictionary<string, NAVService>();
        }

        /*
         * Constructor chained to the default constructor above.
         * @param name The name of the server.
         * 
         * This constructor is the one that should be used most often. 
         */
        public NAVServer(string name) : this()
        {
            serverName = name;            
        }

        /*
         * Method AddService
         * @param service The NAV Service Tier Service to add to our list
         * 
         * Checks to see if the NAV Service Tier service does not already exist in the current list of services.
         * If it does not exist it is added.
         */
        public void AddService(NAVService service)
        {
            if (!NAVServices.ContainsKey(service.DisplayName))
            {
                NAVServices.Add(service.DisplayName, service);
            }
        }

        /*
         * Method LoadNAVServices
         * 
         * Retrieves all of the non-device driver services installed on the server. If it is determined that any 
         * of those services are a NAV Service Tier service then the configuration file is loaded for that service 
         * and it is added to our list of services.
         */
        public void LoadNAVServices()
        {
            ServiceController[] services;
            try
            {
                services = ServiceController.GetServices(serverName);
            }
            catch (Exception e)
            {
                throw new Exception("Unable to retrieve a list of NAV Service Tier Services!" +
                    Environment.NewLine +
                    e.Message);
            }

            foreach (ServiceController sc in services)
            {
                NAVService navService = new NAVService(sc.ServiceName, sc.MachineName);

                if (navService.IsNAVService())
                {
                    navService.LoadConfiguration();
                    AddService(navService);
                }
            }
        }

        /*
         * Function GetService
         * @param name The name of the service to try and retrieve
         * @return NAVService The NAVService object representing the NAV Service Tier service
         * 
         * Retrieves information about a NAV Service Tier service that has been added to our list.
         */
        public NAVService GetService(string name)
        {
            NAVService value;
            if (NAVServices.TryGetValue(name, out value))
            {
                return value;
            }
            else
            {
                return null;
            }            
        }
    }
}
